==> 0.0.4.tar.gz: lib/aargul_local_file_actions.dart <==
part of aargul_local_file;

extension _Unique<E, Id> on List<E> {
  List<E> unique([Id Function(E element)? id, bool inplace = true]) {
    final Set<Id> ids = {};
    var list = inplace ? this : List<E>.from(this);
    list.retainWhere((x) => ids.add(id != null ? id(x) : x as Id));
    return list;
  }
}

class AargulLocalOpennedFile extends AargulModuleState {
  final AargulContext context;
  // final Database _db;
  File? _file;
  AargulLocalOpennedFile(this.context, Database _db, this._file) : super(_db);
  bool get isNew => !(_file?.existsSync() ?? false);
  String? get path => isNew ? null : _file?.path;

  @override
  FutureOr<void> initAction() async {
    await super.initAction();
  }
}

class AargulLocalFileActions extends AargulModuleAction {
  final AargulContext context;
  final AargulLocalFile module;
  late Database _db;
  Stream<List<File>> get opennedFiles =>
      streamItem<String, List<Object?>>("opened", db: _db).transform(
        StreamTransformer.fromHandlers(
          handleData: (value, EventSink<List<File>> sink) {
            if (value != null) {
              sink.add(value.value
                  .map((e) {
                    var f = File(e.toString());
                    return f.existsSync() ? f : null;
                  })
                  .whereType<File>()
                  .toList()); // Duplicate the incoming events.
            }
          },
        ),
      );
  AargulLocalFileActions(this.context, this.module);
  @override
  FutureOr<void> initAction() async {
    _db = await openDatabase(path: "local_file.db");
  }

  Completer<AargulLocalOpennedFile?>? _dialogOpen;

  Future<AargulLocalOpennedFile?> openFile(
      {bool newFile = false, File? existingFile}) async {
    if (_dialogOpen != null) return null;
    _dialogOpen ??= Completer();
    Future.microtask(() async {
      List<File>? file = existingFile != null
          ? [existingFile]
          : (newFile ? null : await _openFile(allowedExtensions: [_extension]));
      if (newFile == true || (file != null && file.isNotEmpty)) {
        var f = AargulLocalOpennedFile(
            context,
            await loadDatabase(newFile
                ? {"sembast_export": 1, "version": 1, "stores": []}
                : jsonDecode(_getStringContent(file!.first.readAsBytesSync()))),
            file?.first);
        if (file != null) {
          await _addFileToHistory(file.first);
        }
        _dialogOpen!.complete(f);
      } else {
        _dialogOpen!.complete(null);
      }
    }).then((value) => _dialogOpen = null).catchError((err) {
      _dialogOpen = null;
    });
    return _dialogOpen!.future;
  }

  Future<AargulLocalOpennedFile> newFile() async {
    return (await openFile(newFile: true))!;
  }

  Future<void> _addFileToHistory(File file) async {
    try {
      if (file.existsSync()) {
        var files =
            (await getItem<String, List<Object?>>("opened", db: _db) ?? [])
                .map((e) => e.toString())
                .toList()
                .unique()
                .reversed
                .toList();
        files.add(file.path);
        await setItem<String, List<Object?>>(
            "opened",
            files
                .toList()
                .reversed
                .whereType<Object?>()
                .toList()
                .sublist(0, min(files.length, 10))
                .unique()
                .toList(),
            db: _db);
      }
    } catch (err) {
      debugPrint("FILES ERROR $err");
    }
  }

  Uint8List _getBinaryContent(String data) {
    var headerTag = Uint8List.fromList(utf8.encode(_extension.toUpperCase()));
    var content = gzip.encode(
      utf8.encode(data),
    );
    ByteData body = ByteData(headerTag.length + content.length + 5);
    body.setInt8(0, headerTag.length); // 1 bytes
    body.setInt32(1, content.length); // 4 bytes
    body.buffer.asUint8ClampedList(5, headerTag.length).setAll(0, headerTag);
    body.buffer.asUint8ClampedList(headerTag.length + 5).setAll(0, content);
    return body.buffer.asUint8List();
  }

  String _getStringContent(Uint8List data) {
    var headerTagLength = data.buffer.asByteData().getInt8(0);
    var contentLength = data.buffer.asByteData().getInt32(1);
    var headerTag = utf8.decode(data.buffer.asUint8List(5, headerTagLength));
    if (_extension.toUpperCase() == headerTag) {
      var content = data.buffer.asUint8List(headerTagLength + 5, contentLength);
      return utf8.decode(gzip.decode(content));
    }
    throw "INVALID FORMAT FILE";
  }

  Future<bool> saveFile(AargulLocalOpennedFile file,
      {String? fileName, String? dialogTitle, String? initialDirectory}) async {
    fileName ??= "new_document";
    if (fileName.isEmpty) {
      fileName = "new_document";
    }
    if (!fileName.toLowerCase().endsWith(".${_extension.toLowerCase()}")) {
      fileName += ".$_extension";
    }
    file._file ??= await _saveFile(
        dialogTitle: dialogTitle,
        initialDirectory: initialDirectory,
        fileName: fileName,
        allowedExtensions: [_extension]);
    if (file._file != null) {
      file._file = await file._file?.writeAsBytes(
        _getBinaryContent(
          jsonEncode(
            await exportDatabase(file.db),
          ),
        ),
      );
      if (file._file != null) {
        await _addFileToHistory(file._file!);
      }
      return true;
    }
    return false;
  }

  Future<File?> _saveFile({
    String? dialogTitle,
    String? fileName,
    String? initialDirectory,
    List<String>? allowedExtensions,
  }) async {
    String? outputFile = await FilePicker.platform.saveFile(
      dialogTitle: dialogTitle,
      fileName: fileName,
      type:
          (allowedExtensions?.isEmpty ?? true) ? FileType.any : FileType.custom,
      allowedExtensions: allowedExtensions,
      initialDirectory: initialDirectory ?? _defaultDir.path,
      lockParentWindow: true,
    );

    if (outputFile == null) {
      // User canceled the picker
      return null;
    }
    return File(outputFile);
  }

  Future<List<File>?> _openFile(
      {bool allowMultiple = false,
      List<String>? allowedExtensions,
      String? dialogTitle,
      String? initialDirectory}) async {
    FilePickerResult? result = await FilePicker.platform.pickFiles(
        allowMultiple: allowMultiple,
        type: (allowedExtensions?.isEmpty ?? true)
            ? FileType.any
            : FileType.custom,
        allowedExtensions: allowedExtensions,
        initialDirectory: initialDirectory ?? _defaultDir.path,
        lockParentWindow: true,
        dialogTitle: dialogTitle);
    if (result != null) {
      return (!allowMultiple ? [result.files.single.path] : result.paths)
          .map((path) => path == null ? null : File(path))
          .whereType<File>()
          .toList();
    } else {
      // User canceled the picker
      return null;
    }
  }
}

==> 0.0.5.tar.gz: lib/aargul_local_file_actions.dart <==
part of aargul_local_file;

extension _Unique<E, Id> on List<E> {
  List<E> unique([Id Function(E element)? id, bool inplace = true]) {
    final Set<Id> ids = {};
    var list = inplace ? this : List<E>.from(this);
    list.retainWhere((x) => ids.add(id != null ? id(x) : x as Id));
    return list;
  }
}

class AargulLocalOpennedFile extends AargulModuleState {
  final AargulContext context;
  // final Database _db;
  File? file;
  AargulLocalOpennedFile(this.context, Database _db, this.file) : super(_db);
  bool get isNew => !(file?.existsSync() ?? false);
  String? get path => isNew ? null : file?.path;

  @override
  FutureOr<void> initAction() async {
    await super.initAction();
  }
}

class AargulLocalFileActions extends AargulModuleAction {
  final AargulContext context;
  final AargulLocalFile module;
  late Database _db;
  Stream<List<File>> get opennedFiles =>
      streamItem<String, List<Object?>>("opened", db: _db).transform(
        StreamTransformer.fromHandlers(
          handleData: (value, EventSink<List<File>> sink) {
            if (value != null) {
              sink.add(value.value
                  .map((e) {
                    var f = File(e.toString());
                    return f.existsSync() ? f : null;
                  })
                  .whereType<File>()
                  .toList()); // Duplicate the incoming events.
            }
          },
        ),
      );
  AargulLocalFileActions(this.context, this.module);
  @override
  FutureOr<void> initAction() async {
    _db = await openDatabase(path: "local_file.db");
  }

  Completer<AargulLocalOpennedFile?>? _dialogOpen;

  Future<AargulLocalOpennedFile?> openFile(
      {bool newFile = false, File? existingFile}) async {
    if (_dialogOpen != null) return null;
    _dialogOpen ??= Completer();
    Future.microtask(() async {
      List<File>? file = existingFile != null
          ? [existingFile]
          : (newFile ? null : await _openFile(allowedExtensions: [_extension]));
      if (newFile == true || (file != null && file.isNotEmpty)) {
        var f = AargulLocalOpennedFile(
            context,
            await loadDatabase(newFile
                ? {"sembast_export": 1, "version": 1, "stores": []}
                : jsonDecode(_getStringContent(file!.first.readAsBytesSync()))),
            file?.first);
        if (file != null) {
          await _addFileToHistory(file.first);
        }
        _dialogOpen!.complete(f);
      } else {
        _dialogOpen!.complete(null);
      }
    }).then((value) => _dialogOpen = null).catchError((err) {
      _dialogOpen = null;
    });
    return _dialogOpen!.future;
  }

  Future<AargulLocalOpennedFile> newFile() async {
    return (await openFile(newFile: true))!;
  }

  Future<void> _addFileToHistory(File file) async {
    try {
      if (file.existsSync()) {
        var files =
            (await getItem<String, List<Object?>>("opened", db: _db) ?? [])
                .map((e) => e.toString())
                .toList()
                .unique()
                .reversed
                .toList();
        files.add(file.path);
        await setItem<String, List<Object?>>(
            "opened",
            files
                .toList()
                .reversed
                .whereType<Object?>()
                .toList()
                .sublist(0, min(files.length, 10))
                .unique()
                .toList(),
            db: _db);
      }
    } catch (err) {
      debugPrint("FILES ERROR $err");
    }
  }

  Uint8List _getBinaryContent(String data) {
    var headerTag = Uint8List.fromList(utf8.encode(_extension.toUpperCase()));
    var content = gzip.encode(
      utf8.encode(data),
    );
    ByteData body = ByteData(headerTag.length + content.length + 5);
    body.setInt8(0, headerTag.length); // 1 bytes
    body.setInt32(1, content.length); // 4 bytes
    body.buffer.asUint8ClampedList(5, headerTag.length).setAll(0, headerTag);
    body.buffer.asUint8ClampedList(headerTag.length + 5).setAll(0, content);
    return body.buffer.asUint8List();
  }

  String _getStringContent(Uint8List data) {
    var headerTagLength = data.buffer.asByteData().getInt8(0);
    var contentLength = data.buffer.asByteData().getInt32(1);
    var headerTag = utf8.decode(data.buffer.asUint8List(5, headerTagLength));
    if (_extension.toUpperCase() == headerTag) {
      var content = data.buffer.asUint8List(headerTagLength + 5, contentLength);
      return utf8.decode(gzip.decode(content));
    }
    throw "INVALID FORMAT FILE";
  }

  Future<bool> saveFile(AargulLocalOpennedFile file,
      {String? fileName, String? dialogTitle, String? initialDirectory}) async {
    fileName ??= "new_document";
    if (fileName.isEmpty) {
      fileName = "new_document";
    }
    if (!fileName.toLowerCase().endsWith(".${_extension.toLowerCase()}")) {
      fileName += ".$_extension";
    }
    file.file ??= await _saveFile(
        dialogTitle: dialogTitle,
        initialDirectory: initialDirectory,
        fileName: fileName,
        allowedExtensions: [_extension]);
    if (file.file != null) {
      file.file = await file.file?.writeAsBytes(
        _getBinaryContent(
          jsonEncode(
            await exportDatabase(file.db),
          ),
        ),
      );
      if (file.file != null) {
        await _addFileToHistory(file.file!);
      }
      return true;
    }
    return false;
  }

  Future<bool> deleteOpenedFile(AargulLocalOpennedFile file) async {
    if (file.file == null) {
      return false;
    }
    return deleteFile(file.file!);
  }

  Future<bool> deleteFile(File file) async {
    if (file.existsSync()) {
      try {
        file.deleteSync();
        List<Object?> files =
            (await getItem<String, List<Object?>>("opened", db: _db) ?? [])
                .map((e) => e.toString())
                .toList()
                .unique()
                .reversed
                .toList();
        files = files
            .toList()
            .reversed
            .whereType<Object?>()
            .toList()
            .sublist(0, min(files.length, 10))
            .unique()
            .toList();
        files.removeWhere(
          (element) => element == null || element.toString() == file.path,
        );
        await setItem<String, List<Object?>>(
          "opened",
          files,
          db: _db,
        );
        return true;
      } catch (e) {
        debugPrint("Error on delete $e");
      }
    }
    return false;
  }

  Future<File?> _saveFile({
    String? dialogTitle,
    String? fileName,
    String? initialDirectory,
    List<String>? allowedExtensions,
  }) async {
    String? outputFile = await FilePicker.platform.saveFile(
      dialogTitle: dialogTitle,
      fileName: fileName,
      type:
          (allowedExtensions?.isEmpty ?? true) ? FileType.any : FileType.custom,
      allowedExtensions: allowedExtensions,
      initialDirectory: initialDirectory ?? _defaultDir.path,
      lockParentWindow: true,
    );

    if (outputFile == null) {
      // User canceled the picker
      return null;
    }
    return File(outputFile);
  }

  Future<List<File>?> _openFile(
      {bool allowMultiple = false,
      List<String>? allowedExtensions,
      String? dialogTitle,
      String? initialDirectory}) async {
    FilePickerResult? result = await FilePicker.platform.pickFiles(
        allowMultiple: allowMultiple,
        type: (allowedExtensions?.isEmpty ?? true)
            ? FileType.any
            : FileType.custom,
        allowedExtensions: allowedExtensions,
        initialDirectory: initialDirectory ?? _defaultDir.path,
        lockParentWindow: true,
        dialogTitle: dialogTitle);
    if (result != null) {
      return (!allowMultiple ? [result.files.single.path] : result.paths)
          .map((path) => path == null ? null : File(path))
          .whereType<File>()
          .toList();
    } else {
      // User canceled the picker
      return null;
    }
  }
}
